#******************************************************************pre_fs.R************************************************************************#
# pre-process the features on the whole data set&do feature selection on the training set
# using rfSA on the training set only
# rm(list=ls())

# import the needed packages
if (!suppressWarnings(require(randomForest))) {
  install.packages("randomForest") 
  require(randomForest)
}
if (!suppressWarnings(require(doParallel))) {
  install.packages("doParallel") 
  require(doParallel)
}
if (!suppressWarnings(require(caret))) {
  install.packages("caret") 
  require(caret)
}
if (!suppressWarnings(require(e1071))) {
  install.packages("e1071") 
  require(e1071)
}
# if (!suppressWarnings(require(klaR))) {
#   install.packages("klaR") 
#   require(klaR)
# }
if (!suppressWarnings(require(dplyr))) {
  install.packages("dplyr") 
  require(dplyr)
}
if (!suppressWarnings(require(pROC))) {
  install.packages("pROC") 
  require(pROC)
}


# get data
bcrp_origin <- read.csv("bcrp_inhibition_all.csv")
bcrp_data <- as.data.frame(bcrp_origin[, c(-1)])  #the first column is the label of data 

# pre-process the features on the whole data set
bcrp_data <- t(na.omit(t(bcrp_data)))  # features with missing values
bcrp_data <- as.data.frame(bcrp_data) 
ZeroVar <- nearZeroVar(bcrp_data,freqCut = 95/5, uniqueCut = 10, names = T)  # features with near zero variance 
deVars1 <- names(bcrp_data) %in% ZeroVar 
bcrp_data <- bcrp_data[!deVars1]

descrCorr <- cor(bcrp_data)
highCorr <- findCorrelation(descrCorr, 0.90, names = T)  # features with high correlation
deVars2 <- names(bcrp_data) %in% highCorr
bcrp_data <- bcrp_data[!deVars2]

# function of normalizing the data
normalize <- function(x){
  return((x - mean(x))/sd(x))
}
bcrp_data <- as.data.frame(lapply(bcrp_data, normalize))

bcrp_data <- cbind(bcrp_origin[, "classification"], bcrp_data)
names(bcrp_data)[1] <- c("classification")
cat("the num of retained features after this step is:", length(names(bcrp_data)) - 1, "\n")

write.csv(bcrp_data, "bcrp_inhibition_all_centralscaled.csv",row.names = T)  # with indexes


# data splitting
# simple splitting based on the outcome
set.seed(1234)
bcrp_data$classification <- factor(bcrp_data[, "classification"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor"))
trainIndex <- createDataPartition(bcrp_data$classification, p = .8, 
                                  list = F, 
                                  times = 1)
bcrp_inhibition_tr <- bcrp_data[trainIndex, ]
# no feature selection on the training set
write.csv(bcrp_inhibition_tr, "bcrp_inhibition_tr_no_fs.csv", row.names = T)  # with indexes
bcrp_inhibition_te <- bcrp_data[-trainIndex, ]
write.csv(bcrp_inhibition_te, "bcrp_inhibition_te_no_fs.csv", row.names = T) # with indexes

num_inbtr <- unname(table(bcrp_data$classification)["inhibitor"])
num_noninbtr <- unname(table(bcrp_data$classification)["noninhibitor"])
cat("the sample distribution in the whole data set is:", "inhibitor:", num_inbtr, "noninhibitor:", num_noninbtr, "ratio:", num_inbtr/num_noninbtr,"\n")

num_inbtr <- unname(table(bcrp_inhibition_tr$classification)["inhibitor"])
num_noninbtr <- unname(table(bcrp_inhibition_tr$classification)["noninhibitor"])
cat("the sample distribution in the training set is:", "inhibitor:", num_inbtr, "noninhibitor:", num_noninbtr, "ratio:", num_inbtr/num_noninbtr,"\n")

num_inbtr <- unname(table(bcrp_inhibition_te$classification)["inhibitor"])
num_noninbtr <- unname(table(bcrp_inhibition_te$classification)["noninhibitor"])
cat("the sample distribution in the test set is:", "inhibitor:", num_inbtr, "noninhibitor:", num_noninbtr, "ratio:", num_inbtr/num_noninbtr,"\n")


# feature selection using rfSA on the training set only
set.seed(1234)
registerDoParallel(12) 
getDoParWorkers()
trainDescr <- bcrp_inhibition_tr[, c(-1)]
train_Y <- bcrp_inhibition_tr$classification
sa_ctrl <- safsControl(functions = rfSA, 
                       method = "repeatedcv",   
                       number = 5,
                       repeats = 5,
                       improve = 50,
                       allowParallel = T)
FeatureSelect <- safs(x = trainDescr, y = train_Y,
                      iters = 1000, #1000 generations of algorithm
                      safsControl = sa_ctrl)

write.csv(FeatureSelect$sa$table, "FeatureSelect$sa$table.csv", row.names = F)
write.csv(as.matrix(FeatureSelect$sa$subsets), "FeatureSelect$sa$subsets.csv", row.names = F)

write.csv(bcrp_inhibition_tr[, c("classification", FeatureSelect$optVariables)], "bcrp_inhibition_tr.csv", row.names = F)  # the training set
write.csv(bcrp_inhibition_te[, c("classification", FeatureSelect$optVariables)], "bcrp_inhibition_te.csv", row.names = F)  # the test set 

write.csv(FeatureSelect$fit$importance, "FeatureSelect$fit$importance.csv", row.names = T)  # get the feature importance 
save.image("FeatureSelect.RData") 
#*******************************************************************pre_fs.R************************************************************************#











#****************************************************************dnn_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

library(h2o)
library(mlrMBO)
library(DiceKriging)
library(pROC)

# initialize h2o
h2o.init(nthreads = -1, min_mem_size = "26G")

# get the data
# bcrp_trainingset <- read.csv("trainingset.csv")
# bcrp_testset <- read.csv("testset.csv")
# bcrp_trainingset$inhibitor <- as.factor(bcrp_trainingset$inhibitor) 
# bcrp_testset$inbitor <- as.factor(bcrp_testset$inhibitor) 

bcrp_trainingset <- read.csv("bcrp_inhibition_tr.csv", stringsAsFactors = F)
bcrp_trainingset$classification <- as.integer(ifelse(bcrp_trainingset$classification == "inhibitor", 1, 0))
bcrp_trainingset$classification <- as.factor(bcrp_trainingset$classification)

bcrp_testset <- read.csv("bcrp_inhibition_te.csv",stringsAsFactors = F)
bcrp_testset$classification <- as.integer(ifelse(bcrp_testset$classification == "inhibitor", 1, 0))
bcrp_testset$classification <- as.factor(bcrp_testset$classification)

bcrp_trainingset.hex <- as.h2o(x = bcrp_trainingset, destination_frame = "bcrp_trainingset.hex")
bcrp_testset.hex <- as.h2o(x = bcrp_testset, destination_frame = "bcrp_testset.hex")

# for test the code 
# bcrp_trainingset.hex <- bcrp_trainingset.hex[1:200,]
# bcrp_testset.hex <- bcrp_testset.hex[1:100,]

# method name
method.name <- "deeplearning"

# hyper-parametres
parameters <- c("mini_batch_size", "l2", "rho", "epsilon", "input_dropout_ratio", "hidden_dropout_ratios", "hidden_layers", "hidden_layer_units")


# /*****************Beyesian optimization****************************/
# target function
# dnn_boa_info <<- as.data.frame(array(dim = c(0,length(parameters)+2)))  # Global Environment

fun <- function(para){
  Model <- h2o.deeplearning(y = "classification", activation = "RectifierWithDropout", training_frame = bcrp_trainingset.hex, nfolds = 5, epochs = 300, 
                            fold_assignment = "Stratified", score_training_samples = 0, overwrite_with_best_model = T, standardize = F, 
                            shuffle_training_data = T, seed = 1, stopping_metric = "misclassification", stopping_tolerance = 0.001, keep_cross_validation_models = F,
                            stopping_rounds = 5, hidden = rep(unname(para[8]), unname(para[7])), mini_batch_size = unname(para[1]), l2 = unname(para[2]), rho = unname(para[3]), epsilon = unname(para[4]), 
                            input_dropout_ratio = unname(para[5]), hidden_dropout_ratios = rep(unname(para[6]), unname(para[7])));
  
  threshold.rev <- rev(Model@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$threshold); 
  MCC.rev <- rev(Model@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$absolute_mcc);
  for (i in 1:length(threshold.rev)) {
    if (threshold.rev[i] >= 0.5) break;
  };
  
  h2o.saveModel(Model, path = getwd(), force = T);
  
  results.parameters <- c(para[1], para[2], para[3], para[4], para[5], para[6], para[7], para[8], Model@model_id, 1 - MCC.rev[i]);
  results.parameters_add <- c(0, 0, 0, 0, 0, 0, 0, 0, "0", 0);
  results <- rbind(results.parameters,results.parameters_add);
  colnames(results) <- c(parameters[1], parameters[2], parameters[3], parameters[4], parameters[5], parameters[6], parameters[7], 
                         parameters[8], "model_id", "y");
  write.table(results,"dnn_boa_info.csv", append = T, row.names = T, col.names = NA, sep = ",");
  
  #avoiding of out of memory 
  h2o.rm(Model)
  
  1 - MCC.rev[i] 
}


objfun <- makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""),
  fn = fun,
  par.set = makeParamSet(  # hyper-parametres optimization space
    makeIntegerParam(parameters[1], lower = 1, upper = 32), 
    makeNumericParam(parameters[2], lower = 0, upper = 0.01), 
    makeNumericParam(parameters[3], lower = 0.9, upper = 0.999),
    makeNumericParam(parameters[4], lower = 1e-10, upper = 1e-7),
    makeNumericParam(parameters[5], lower = 0.0, upper = 0.2),
    makeNumericParam(parameters[6], lower = 0.0, upper = 0.3),
    makeIntegerParam(parameters[7], lower = 2, upper = 5), 
    makeIntegerParam(parameters[8], lower = 100, upper = 300) 
    
  ),
  minimize = T
)

# sorrogate Learner
surr.km <- makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

# infill Criterion(aka. acquisition functions)
control <- makeMBOControl(save.on.disk.at = seq(from = 0, to = 475, by = 25))
control <- setMBOControlInfill(
  control = control,
  crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

# termination condition
control <- setMBOControlTermination(
  control = control,
  iters = 500  # 500 iterations
)

# initial Design
# set.seed(16)
design <- generateDesign(n = 200, par.set = getParamSet(objfun)) 


# optimization
# surpresses output of learners
mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")

# if mbo() success, class(try_res) = c("MBOSingleObjResult" "MBOResult"), else, class(try_res) = "try-error"
try_res <- try(mbo(objfun, design = design, learner = surr.km, control = control, show.info = T), silent = F)
if (class(try_res) == "try-error") {
  run <- mboContinue(file.path(getwd(), "mlr_run.RData"))
} else {
  run <- try_res
}

# names(dnn_boa_info) <- c(parameters[1], parameters[2], parameters[3], parameters[4], parameters[5], parameters[6], parameters[7], parameters[8], "model_id", "y")
# write.csv(dnn_boa_info, "dnn_boa_info.csv");

# /**********************Beyesian optimization over**********************/

# modeling according to the optimal hyper-parametres determined by Bayesian optimization
# reload the best dnn model 
# info <- dnn_boa_info
# info <- read.csv("dnn_boa_info.csv")
# path <- paste(getwd(), '/', info$model_id[which(info$y == run$y)], sep = '')
# bestModel <- h2o.loadModel(path)

bestModel <- h2o.deeplearning(x = x, y = y, activation = "RectifierWithDropout", training_frame = bcrp_trainingset.hex, nfolds = 5, epochs = 300,
                              fold_assignment = "Stratified", score_training_samples = 0, overwrite_with_best_model = T, standardize = F, shuffle_training_data = T, keep_cross_validation_predictions = T, keep_cross_validation_fold_assignment = T,
                              seed = 1, stopping_metric = "misclassification", stopping_tolerance = 0.001, stopping_rounds = 5, hidden = rep(run$x$hidden_layer_units, run$x$hidden_layers),
                              mini_batch_size = run$x$mini_batch_size, l2 = run$x$l2, rho = run$x$rho, epsilon = run$x$epsilon, input_dropout_ratio = run$x$input_dropout_ratio,
                              hidden_dropout_ratios = rep(run$x$hidden_dropout_ratios, run$x$hidden_layers));

# training_metrics
train.threshold.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$threshold);
for (i in 1:length(train.threshold.rev)) {
  if (train.threshold.rev[i] >= 0.5) break;
}
train.TP.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$tps);
train.FP.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$fps);
train.TN.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$tns);
train.FN.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$fns);
train.SE.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$recall);
train.SP.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$specificity);
train.Accuracy.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$accuracy);
train.MCC.rev <- rev(bestModel@model$training_metrics@metrics$thresholds_and_metric_scores$absolute_mcc);

train.TP <- train.TP.rev[i];train.FP <- train.FP.rev[i];train.TN <- train.TN.rev[i];train.FN <- train.FN.rev[i];
train.SE <- train.SE.rev[i];train.SP <- train.SP.rev[i];train.Accuracy <- train.Accuracy.rev[i];train.MCC <- train.MCC.rev[i];
train.AUC <- bestModel@model$training_metrics@metrics$AUC; 

# cross_validation_metrics
train_cv.threshold.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$threshold);
for (i in 1:length(train_cv.threshold.rev)) {
  if (train_cv.threshold.rev[i] >= 0.5) break;
}
train_cv.TP.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$tps);
train_cv.FP.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$fps);
train_cv.TN.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$tns);
train_cv.FN.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$fns);
train_cv.SE.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$recall);
train_cv.SP.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$specificity);
train_cv.Accuracy.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$accuracy);
train_cv.MCC.rev <- rev(bestModel@model$cross_validation_metrics@metrics$thresholds_and_metric_scores$absolute_mcc);

train_cv.TP <- train_cv.TP.rev[i];train_cv.FP <- train_cv.FP.rev[i];train_cv.TN <- train_cv.TN.rev[i];train_cv.FN <- train_cv.FN.rev[i];
train_cv.SE <- train_cv.SE.rev[i];train_cv.SP <- train_cv.SP.rev[i];train_cv.Accuracy <- train_cv.Accuracy.rev[i];train_cv.MCC <- train_cv.MCC.rev[i];
train_cv.AUC <- bestModel@model$cross_validation_metrics@metrics$AUC 

# test set metrics
test_pred <- as.data.frame(h2o.predict(bestModel, newdata = bcrp_testset.hex)) 
test_pred$new_pred[test_pred["p1"] >= test_pred["p0"]] <- 2
test_pred$new_pred[test_pred["p1"] < test_pred["p0"]] <- 1 

test.tab <- table(test_pred$new_pred, as.numeric(bcrp_testset$classification))
test.TP <- test.tab[2,2];test.FP <- test.tab[2,1];test.FN <- test.tab[1,2];test.TN <- test.tab[1,1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.Accuracy <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test.AUC <- roc(as.numeric(bcrp_testset$classification) - 1,test_pred$p1)$auc[1]

# save results
results.train_cv <- c(run$x$mini_batch_size, run$x$l2, run$x$rho, run$x$epsilon, run$x$input_dropout_ratio, run$x$hidden_dropout_ratios, run$x$hidden_layers, 
                      run$x$hidden_layer_units, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.Accuracy, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$mini_batch_size, run$x$l2, run$x$rho, run$x$epsilon, run$x$input_dropout_ratio, run$x$hidden_dropout_ratios, run$x$hidden_layers, 
                   run$x$hidden_layer_units, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.Accuracy, train.MCC, train.AUC);
results.test <- c(run$x$mini_batch_size, run$x$l2, run$x$rho, run$x$epsilon, run$x$input_dropout_ratio, run$x$hidden_dropout_ratios, run$x$hidden_layers, 
                  run$x$hidden_layer_units, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.Accuracy, test.MCC, test.AUC);

results <- rbind(results.train, results.train_cv, results.test);
colnames(results) <- c(parameters[1], parameters[2], parameters[3], parameters[4], parameters[5], parameters[6], parameters[7], parameters[8], 
                       "TP", "FP", "FN", "TN", "SE", "SP", "Accuracy", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T) 

# save the best dnn model
h2o.saveModel(bestModel, path = getwd(), force = T);

cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n"); 

# save the working space
save.image(paste(attributes(objfun)$name, ".RData", sep = ""))


#****************************************************************dnn_modeling.R*********************************************************************#










#****************************************************************knn_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

# import the needed packages
library(kknn)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)

# get the data
# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor"))

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

# parallel processing
# cl <- makePSOCKcluster(6) 
cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

# set seeds to guarantee the repeatability of the model (used for parallel processing)
set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1) 
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

# method name
method.name <- "kknn";
# hyper-parametres
parameters <- c("kmax", "distance", "kernel");

# /*****************Beyesian optimization****************************/
# target function
fun <- function(x){
  set.seed(k)	
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(kmax = x$kmax, distance = x$distance, kernel = x$kernel))
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
  1 - train_cv.MCC 
  # 1-roc(Model$pred$obs, Model$pred$inhibitor)$auc[1]; 
}
objfun = makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(  # optimization space settings
    makeIntegerParam(parameters[1], lower = 3, upper = 15), 
    makeNumericParam(parameters[2], lower = 1, upper = 3), 
    makeDiscreteParam(parameters[3], values = c("triangular","rectangular","epanechnikov","optimal","biweight","triweight","cos","inv","gaussian","rank" ))
  ), 
  has.simple.signature = F, 
  minimize = T
)

# sorrogate Learner
surr.rf = makeLearner("regr.randomForest", predict.type = "se") 
# surr.km = makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

# infill Criterion(aka. acquisition functions)
control = makeMBOControl()
control = setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritCB(cb.lambda = 5), 
  # crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

# termination condition
control = setMBOControlTermination(
  control = control, 
  iters = 500 # 500 iterations
)

# initial Design
set.seed(4)
design = generateDesign(n = 200, par.set = getParamSet(objfun)) 

# optimization
#  surpresses output of learners
mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run = mbo(objfun, design = design, learner = surr.rf, control = control, show.info = T)
# /**********************Beyesian optimization**********************/

# modeling according to the optimal hyper-parametres determined by Bayesian optimization
set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(kmax = run$x$kmax, distance = run$x$distance,  kernel = run$x$kernel))

train_cv.tab <- table(bestModel$pred$pred,  bestModel$pred$obs)
train_cv.TP <- train_cv.tab[2,  2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN  +  test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$kmax, run$x$distance, run$x$kernel, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$kmax, run$x$distance, run$x$kernel, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$kmax, run$x$distance, run$x$kernel, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);colnames(results) <- c(parameters[1], parameters[2], parameters[3], "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T)

# stopCluster(cl)
cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 
save.image(paste(attributes(objfun)$name, ".RData", sep = ""))
#****************************************************************knn_modeling.R*********************************************************************#










#****************************************************************nb_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

# import the needed packages
library(naivebayes)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)


# get the data
# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

#parallel processing
#cl <- makePSOCKcluster(6)
cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

#set seeds to guarantee the repeatability of the model (used for parallel processing)
set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1) 
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

# method name
method.name <- "naive_bayes";

# hyper-parametres
parameters <- c("laplace", "usekernel", "adjust");

#/*****************Bayesian optimization****************************/
# target function
fun  <- function(x){
  set.seed(k);
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(laplace = x$laplace, usekernel = x$usekernel, adjust = x$adjust));
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
  1 - train_cv.MCC  # based on the MCC value of the 5-fold cross-validation to the training set
}
objfun <- makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(  # optimization space settings
    makeNumericParam(parameters[1], lower = 0.1, upper = 10), 
    # makeDiscreteParam(parameters[2], values = c(TRUE, FALSE)),  # mixed space
    makeLogicalParam(parameters[2]),
    makeNumericParam(parameters[3], lower = 0.1, upper = 10)
  ), 
  has.simple.signature = F, 
  minimize = T
)

# sorrogate Learner
surr.rf <- makeLearner("regr.randomForest", predict.type = "se")

# infill Criterion(aka. acquisition functions)
control <- makeMBOControl()
control <- setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritCB(cb.lambda = 5), 
  opt.focussearch.points = 500
)

# termination condition
control <- setMBOControlTermination(
  control = control, 
  iters = 500
)

# initial Design
design <- generateDesign(n = 200, par.set = getParamSet(objfun))

# optimization
# surpresses output of learners
mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run <- mbo(objfun, design = design, learner = surr.rf, control = control, show.info = T)
# /**********************Bayesian optimization over******************/

# modeling according to the optimal hyper-parametres determined by Bayesian optimization
set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(laplace  = run$x$laplace, usekernel  = run$x$usekernel, adjust = run$x$adjust))

train_cv.tab <- table(bestModel$pred$pred, bestModel$pred$obs)  # statisticals of 5-fold cross-validated training set
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);  # statisticals of the whole training set
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);  # statisticals of the test set
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$laplace, run$x$usekernel, run$x$adjust, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$laplace, run$x$usekernel, run$x$adjust, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$laplace, run$x$usekernel, run$x$adjust, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);colnames(results) <- c(parameters[1], parameters[2], parameters[3], "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T) 

cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 
save.image(paste(attributes(objfun)$name, ".RData", sep = ""))
#****************************************************************nb_modeling.R*********************************************************************#










#****************************************************************rlr_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

library(LiblineaR)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)

# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor"))
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor"))

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

# cl <- makePSOCKcluster(6) 
cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1) 
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

method.name <- "regLogistic";

parameters <- c("cost", "loss", "epsilon");

fun <- function(x){
  set.seed(k)	
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                        tuneGrid = data.frame(cost = x$cost, loss = x$loss, epsilon = x$epsilon))
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP  +  train_cv.FP) %*% (train_cv.TP  +  train_cv.FN) %*% (train_cv.TN  +  train_cv.FP) %*% (train_cv.TN  +  train_cv.FN)));
  1 - train_cv.MCC 
  # 1-roc(Model$pred$obs, Model$pred$inhibitor)$auc[1]; 
}
objfun <- makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(
    makeNumericParam(parameters[1], lower = 1, upper = 10), 
    makeDiscreteParam(parameters[2], values = c("L1", "L2_dual", "L2_primal")), 
    makeNumericParam(parameters[3], lower = 0.1, upper = 1)
    
  ), 
  has.simple.signature =  F, 
  minimize =  T
)

surr.rf <- makeLearner("regr.randomForest", predict.type = "se") 
# surr.km = makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

control <- makeMBOControl()
control <- setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritCB(cb.lambda = 5), 
  # crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

control <- setMBOControlTermination(
  control = control, 
  iters = 500 
)

set.seed(4)
design <- generateDesign(n = 200, par.set = getParamSet(objfun))

mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run <- mbo(objfun, design = design, learner = surr.rf, control = control, show.info = T)

set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                          tuneGrid = data.frame(cost  = run$x$cost, loss  = run$x$loss, epsilon = run$x$epsilon))

train_cv.tab <- table(bestModel$pred$pred, bestModel$pred$obs)
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP  +  train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN  +  train_cv.FP);train_cv.GA <- (train_cv.TP  +  train_cv.TN)/(train_cv.TP  +  train_cv.FN  +  train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP  +  test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$cost, run$x$loss, run$x$epsilon, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$cost, run$x$loss, run$x$epsilon, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$cost, run$x$loss, run$x$epsilon, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);colnames(results) <- c(parameters[1], parameters[2], parameters[3], "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T) 

# stopCluster(cl)
cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 
save.image(paste(attributes(objfun)$name, ".RData", sep = ""))

#****************************************************************rlr_modeling.R*********************************************************************#










#****************************************************************sgb_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

library(gbm)
library(plyr)
library(DiceKriging)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)

# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

cl = 6
registerDoParallel(cl) 
getDoParWorkers()

set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1) 
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

method.name = "gbm";

parameters <- c("n.trees", "interaction.depth", "n.minobsinnode", "shrinkage");

fun  <- function(x){
  set.seed(k);
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                        tuneGrid = data.frame(n.trees = unname(x[1]), interaction.depth = unname(x[2]), n.minobsinnode = unname(x[3]), shrinkage = unname(x[4])));
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
  1 - train_cv.MCC 
}
objfun = makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(
    makeIntegerParam(parameters[1], lower = 160, upper = 200), 
    makeIntegerParam(parameters[2], lower = 20, upper = 35), 
    makeIntegerParam(parameters[3], lower = 20, upper = 30), 
    makeNumericParam(parameters[4], lower = 0.001, upper = 0.1)
  ), 
  minimize = T
)

surr.km = makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

control = makeMBOControl()
control = setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

control = setMBOControlTermination(
  control = control, 
  iters = 500
)

design = generateDesign(n = 200, par.set = getParamSet(objfun))

mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run = mbo(objfun, design = design, learner = surr.km, control = control, show.info = T)

set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                          tuneGrid = data.frame(n.trees = run$x$n.trees, interaction.depth = run$x$interaction.depth, n.minobsinnode = run$x$n.minobsinnode, shrinkage = run$x$shrinkage));

train_cv.tab <- table(bestModel$pred$pred, bestModel$pred$obs);
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$n.trees, run$x$interaction.depth, run$x$n.minobsinnode, run$x$shrinkage, 
                      train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$n.trees, run$x$interaction.depth, run$x$n.minobsinnode, run$x$shrinkage, 
                   train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$n.trees, run$x$interaction.depth, run$x$n.minobsinnode, run$x$shrinkage
                  , test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);
colnames(results) <- c(parameters[1], parameters[2], parameters[3], parameters[4], 
                       "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T)

cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 

save.image(paste(attributes(objfun)$name, ".RData", sep = ""))

#****************************************************************sgb_modeling.R*********************************************************************#











#****************************************************************svm_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

library(kernlab)
library(DiceKriging)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)

# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1) 
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

method.name <- "svmRadial";

parameters <- c("sigma", "C");

fun <- function(x){
  set.seed(k);
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(sigma = unname(x[1]), C = unname(x[2])));
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
  1 - train_cv.MCC 
}
objfun <- makeSingleObjectiveFunction(
  name <- paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(
    makeNumericParam(parameters[1], lower = 0.005, upper = 0.02), 
    makeNumericParam(parameters[2], lower = 5, upper = 20)
  ), 
  minimize = T
)

surr.km <- makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

control <- makeMBOControl()
control <- setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

control <- setMBOControlTermination(
  control = control, 
  iters = 500
)

design <- generateDesign(n = 200, par.set = getParamSet(objfun))

mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run <- mbo(objfun, design = design, learner = surr.km, control = control, show.info = T)


set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), tuneGrid = data.frame(C = run$x$C, sigma = run$x$sigma));

train_cv.tab <- table(bestModel$pred$pred, bestModel$pred$obs);
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$C, run$x$sigma, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$C, run$x$sigma, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$C, run$x$sigma, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);colnames(results) <- c(parameters[2], parameters[1], "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T) 

cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 
save.image(paste(attributes(objfun)$name, ".RData", sep = ""))

#****************************************************************svm_modeling.R*********************************************************************#











#****************************************************************xgb_modeling.R*********************************************************************#
ptm <- unclass(Sys.time())

library(xgboost)
library(plyr)
library(DiceKriging)
library(caret)
library(mlrMBO)
library(pROC)
library(e1071)
library(doParallel)

# trainingset <- read.csv("trainingset.csv")
# train_x <- trainingset[, c(-1)]
# train_y <- factor(trainingset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 
# testset <- read.csv("testset.csv")
# test_x <- testset[, c(-1)]
# test_y <- factor(testset[, "inhibitor"], levels = c(0, 1), labels = c("noninhibitor", "inhibitor")) 

trainingset <- read.csv("bcrp_inhibition_tr.csv")
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 
testset <- read.csv("bcrp_inhibition_te.csv")
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1)
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

method.name <- "xgbTree";

parameters <- c("nrounds", "max_depth", "eta", "gamma", "colsample_bytree", "min_child_weight", "subsample");

fun  <- function(x){
  set.seed(k);
  Model <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                        tuneGrid = data.frame(nrounds = x[1], max_depth = x[2], eta = x[3], gamma = x[4], colsample_bytree = x[5], min_child_weight = x[6], subsample = x[7]));
  train_cv.tab <- table(Model$pred$pred, Model$pred$obs);
  train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
  train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
  1 - train_cv.MCC 
}
objfun <- makeSingleObjectiveFunction(
  name = paste(method.name, "_boa", sep = ""), 
  fn = fun, 
  par.set = makeParamSet(
    makeIntegerParam(parameters[1], lower = 200, upper = 300), 
    makeIntegerParam(parameters[2], lower = 9, upper = 11), 
    makeNumericParam(parameters[3], lower = 0.01, upper = 0.1), 
    makeNumericParam(parameters[4], lower = 0.4, upper = 0.6), 
    makeNumericParam(parameters[5], lower = 0.4, upper = 0.6), 
    makeNumericParam(parameters[6], lower = 0.7, upper = 0.9), 
    makeNumericParam(parameters[7], lower = 0.7, upper = 0.9)
  ), 
  minimize = T
)

surr.km <- makeLearner("regr.km", predict.type = "se", covtype = "matern3_2", control = list(trace = F)) 

control <- makeMBOControl()
control <- setMBOControlInfill(
  control = control, 
  crit = makeMBOInfillCritEI(), 
  opt.focussearch.points = 500
)

control <- setMBOControlTermination(
  control = control, 
  iters = 500
)

design <- generateDesign(n = 200, par.set = getParamSet(objfun))

mlr::configureMlr(show.info = F, show.learner.output = F, on.learner.warning = "quiet")
run <- mbo(objfun, design = design, learner = surr.km, control = control, show.info = T)

set.seed(k)
bestModel <- caret::train(train_x, train_y, method = method.name, trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                          tuneGrid = data.frame(nrounds = run$x$nrounds, max_depth = run$x$max_depth, eta = run$x$eta, gamma = run$x$gamma, 
                                                colsample_bytree = run$x$colsample_bytree, min_child_weight = run$x$min_child_weight, subsample = run$x$subsample));

train_cv.tab <- table(bestModel$pred$pred, bestModel$pred$obs);
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(bestModel$pred$obs, bestModel$pred$inhibitor)$auc[1]

train.pred <- predict(bestModel, train_x);train.tab <- table(train.pred, train_y);
train.TP <- train.tab[2, 2];train.FP <- train.tab[2, 1];train.FN <- train.tab[1, 2];train.TN <- train.tab[1, 1];
train.SE <- train.TP/(train.TP + train.FN);train.SP <- train.TN/(train.TN + train.FP);train.GA <- (train.TP + train.TN)/(train.TP + train.FN + train.TN + train.FP);
train.MCC <- as.numeric((train.TP*train.TN - train.FP*train.FN)/sqrt((train.TP + train.FP) %*% (train.TP + train.FN) %*% (train.TN + train.FP) %*% (train.TN + train.FN)));
train_predictions <- predict(bestModel, train_x, type = "prob") 
train.AUC <- roc(train_y, train_predictions$inhibitor)$auc[1] 

test.pred <- predict(bestModel, test_x);test.tab <- table(test.pred, test_y);
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test_predictions <- predict(bestModel, test_x, type = "prob") 
test.AUC <- roc(test_y, test_predictions$inhibitor)$auc[1] 

results.train_cv <- c(run$x$nrounds, run$x$max_depth, run$x$eta, run$x$gamma, run$x$colsample_bytree, run$x$min_child_weight, 
                      run$x$subsample, train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.train <- c(run$x$nrounds, run$x$max_depth, run$x$eta, run$x$gamma, run$x$colsample_bytree, run$x$min_child_weight, 
                   run$x$subsample, train.TP, train.FP, train.FN, train.TN, train.SE, train.SP, train.GA, train.MCC, train.AUC);
results.test <- c(run$x$nrounds, run$x$max_depth, run$x$eta, run$x$gamma, run$x$colsample_bytree, run$x$min_child_weight, 
                  run$x$subsample, test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train, results.train_cv, results.test);
colnames(results) <- c(parameters[1], parameters[2], parameters[3], parameters[4], parameters[5], parameters[6], parameters[7], 
                       "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results, paste(attributes(objfun)$name, "_results.csv", sep = ""), row.names = T) 

cat("elapsed time is:", (unclass(Sys.time()) - ptm), "s \n") 

save.image(paste(attributes(objfun)$name, ".RData", sep = ""))

#****************************************************************xgb_modeling.R*********************************************************************#










#****************************************************************pca_analysis.R*********************************************************************#
# for pca analysis
library(psych)
library(ggplot2)
library(gridExtra)

# prepare data
bcrp_tr <- read.csv("bcrp_inhibition_tr.csv")
bcrp_tr$group <- "Training Set"
bcrp_te <- read.csv("bcrp_inhibition_te.csv")
bcrp_te$group <- "Test Set"

# Weight & SlogP
bcrp_tr_no_fs <- read.csv("bcrp_inhibition_tr_no_fs.csv")
train_indx <- bcrp_tr_no_fs$X
bcrp_all <- read.csv("bcrp_inhibition_all.csv")
bcrp_ws_tr <- bcrp_all[,c("Weight","SlogP")][train_indx,]
bcrp_ws_te <- bcrp_all[,c("Weight","SlogP")][-train_indx,]
bcrp_ws <- rbind(bcrp_ws_tr, bcrp_ws_te)

bcrp_pca <- rbind(bcrp_tr, bcrp_te)
myvars <- names(bcrp_pca) %in% c("classification","group")
# bcrp_pca <- bcrp_pca[!myvars]

# 
pca_scores <- principal(bcrp_pca[!myvars], nfactors = 3, rotate = 'none', score = T)
pca_res <- cbind(pca_scores$scores, bcrp_ws, bcrp_pca$group, stringsAsFactors = F)
names(pca_res)[length((names(pca_res)))] <- "group"
# pca_res$group <- factor(pca_res$group)
pca_res$group <- factor(pca_res$group, levels = c("Training Set","Test Set"))

# plotting

# scale_color_manual(values = c("blue","red")) # custom color for group
# axis.title.x, the title for x axis
# axis.title.y, the title for y axis
# axis.text.x, tick label for x axis

title_style <- element_text(size = 10, face = "bold")
tick_style <- element_text(size = 8, face = "bold")


p1 <- ggplot(data = pca_res, aes(x = PC1, y = PC2, colour = group)) + geom_point(size = 1.0) + 
  theme(axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.direction = "horizontal", legend.title = element_blank(), legend.position = "top", legend.key = element_blank(),legend.background = element_blank())

p2 <- ggplot(data = pca_res, aes(x = PC1, y = PC3, colour = group)) + geom_point(size = 1.0) + 
  theme(axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.direction = "horizontal", legend.title = element_blank(), legend.position = "top", legend.key = element_blank(),legend.background = element_blank())

p3 <- ggplot(data = pca_res, aes(x = PC2, y = PC3, colour = group)) + geom_point(size = 1.0) + 
  theme(axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.direction = "horizontal", legend.title = element_blank(), legend.position = "top", legend.key = element_blank(),legend.background = element_blank())

p4 <- ggplot(data = pca_res, aes(x = Weight, y = SlogP, colour = group)) + geom_point(size = 1.0) + 
  theme(axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.direction = "horizontal", legend.title = element_blank(), legend.position = "top", legend.key = element_blank(),legend.background = element_blank()) + labs(x = "Molecular Weight", y = "SlogP") 
myplot <- grid.arrange(p1, p2, p3, p4, ncol = 2, nrow = 2)      
ggsave('chemical_space.tiff', device = "tiff", plot = myplot, dpi = 300, width = 7, height = 6)
#****************************************************************pca_analysis.R*********************************************************************#










#****************************************************************model_interpretation.R*********************************************************************#
# import the needed packages
# shared
library(caret)
library(e1071)
library(doParallel)
library(pROC)
library(DALEX)
library(gridExtra)

# DNN, k-NN, NB, RLR, SGB, XGBoost, SVM
library(h2o)
library(kknn)
library(naivebayes)
library(LiblineaR)
library(gbm)
library(xgboost)
library(kernlab)

# **********************************
# residual distribution exploration
# **********************************
# for the DNN modeling
# correct names
# cor_names <- function(dataset){
#   names(dataset)[names(dataset) == "PEOE_VSA.6"] = "PEOE_VSA+6"
#   names(dataset)[names(dataset) == "PEOE_VSA.4"] = "PEOE_VSA+4"
#   names(dataset)[names(dataset) == "PEOE_VSA.3.1"] = "PEOE_VSA-3"
#   names(dataset)[names(dataset) == "PEOE_VSA.3"] = "PEOE_VSA+3"
#   names(dataset)[names(dataset) == "PEOE_VSA.2"] = "PEOE_VSA+2"
#   names(dataset)[names(dataset) == "PEOE_VSA.1.1"] = "PEOE_VSA-1"
#   names(dataset)[names(dataset) == "PEOE_VSA.0.1"] = "PEOE_VSA-0"
#   names(dataset)[names(dataset) == "PEOE_RPC."] = "PEOE_RPC+"
#   names(dataset)[names(dataset) == "logP.o.w."] = "logP(o/w)"
#   names(dataset)[names(dataset) == "FASA..1"] = "FASA-"
#   names(dataset)[names(dataset) == "ASA..1"] = "ASA-"
#   return(dataset)
# }


h2o.init()  # initialize h2o

# training set
bcrp_trainingset <- read.csv("bcrp_inhibition_tr.csv", stringsAsFactors = F)
# bcrp_trainingset <- cor_names(bcrp_trainingset)
bcrp_trainingset$classification <- as.integer(ifelse(bcrp_trainingset$classification == "inhibitor", 1, 0))
bcrp_trainingset$classification <- as.factor(bcrp_trainingset$classification)
bcrp_trainingset.hex <- as.h2o(x = bcrp_trainingset, destination_frame = "bcrp_trainingset.hex")

# DNN modeling
# bestDNNmodel <- h2o.deeplearning(y = "classification", activation = "RectifierWithDropout", training_frame = bcrp_trainingset.hex, nfolds = 5, epochs = 300, keep_cross_validation_models = F,
#                               keep_cross_validation_predictions = T, keep_cross_validation_fold_assignment = T,fold_assignment = "Stratified", score_training_samples = 0, 
#                               overwrite_with_best_model = T, standardize = F, shuffle_training_data = T, seed = 1, stopping_metric = "misclassification", stopping_tolerance = 0.001, 
#                               stopping_rounds = 5, hidden = rep(265, 3), mini_batch_size = 2, l2 = 0.000896871285824059, rho = 0.964178441767299, epsilon = 6.65330782929858e-09, 
#                               input_dropout_ratio = 0.0672305919617168, hidden_dropout_ratios = rep(0.139116530380509, 3));
# DeepLearning_model_R_1568964995909_1

# get the best DNN model
bestDNNmodel <- h2o.loadModel(file.path(getwd(),"DeepLearning_model_R_1568964995909_1"))

# custom prediction function(for DNN)
custom_predict <- function(model, newdata) {
  newdata = as.h2o(newdata)
  res = as.data.frame(h2o.predict(model, newdata))
  res = res[, c("p0", "p1")]
  return(res)
}

# custom residual function (cross entropy, for DNN)
residual_function <- function(model, newdata, y){
  # newdata$classification = as.integer(ifelse(newdata$classification == "inhibitor", 1, 0))
  # newdata$classification = as.factor(newdata$classification)
  newdata = as.h2o(newdata)
  res = as.data.frame(h2o.predict(model, newdata))
  res = res[, c("p0", "p1")]
  
  p <- sapply(seq_along(y), function(i) res[i, y[i]])
  return(-log(pmax(p, 1e-4)))
}

# custom model performance function(for all model)
model_performance_ <- function(explainer, ...) 
{
  if (!("explainer" %in% class(explainer))) 
    stop("The model_performance() function requires an object created with explain() function.")
  if (is.null(explainer$data)) 
    stop("The model_performance() function requires explainers created with specified 'data' parameter.")
  if (is.null(explainer$y)) 
    stop("The model_performance() function requires explainers created with specified 'y' parameter.")
  if (is.null(explainer$y_hat)) {
    predicted <- explainer$predict_function(explainer$model, 
                                            explainer$data, ...)
  }
  else {
    predicted <- explainer$y_hat
  }
  observed <- explainer$y
  if (is.null(explainer$residuals)) {
    diff <- predicted - observed
  }
  else {
    diff <- explainer$residuals  #changed here
    # diff <- -explainer$residuals  #origin
  }
  residuals <- data.frame(predicted, observed, diff = diff)
  class(residuals) <- c("model_performance_explainer", "data.frame")
  residuals$label <- explainer$label
  residuals
}

# DNN explainer
# load the test set
bcrp_testset <- read.csv("bcrp_inhibition_te.csv",stringsAsFactors = F)
# bcrp_testset <- cor_names(bcrp_testset)
bcrp_testset$classification <- as.integer(ifelse(bcrp_testset$classification == "inhibitor", 1, 0))
bcrp_testset$classification <- as.factor(bcrp_testset$classification)

explainer_dnn <- DALEX::explain(model = bestDNNmodel, 
                                data = bcrp_testset[,c(-1)],  
                                y = as.numeric(bcrp_testset$classification), 
                                predict_function = custom_predict,
                                residual_function = residual_function,
                                label = "dnn")

# model performance
mp_dnn <- model_performance_(explainer_dnn) #cross entropy distribution for each observation in the test set 
names(mp_dnn) <- c("noninhibitor","inhibitor","observed","diff","label" )



# *************
# other models 
# *************

# custom prediction function (except for dnn)
custom_predict_ <- function(model, newdata) {
  res = predict(model, newdata, type = "prob")
  return(res)
}

# custom residual function (cross entropy, except for dnn)
residual_function_ <- function(model, newdata, y){
  res = predict(model, newdata, type = "prob")
  
  p <- sapply(seq_along(y), function(i) res[i, y[i]])
  return(-log(pmax(p, 1e-4)))
}

# load the data set
trainingset <- read.csv("bcrp_inhibition_tr.csv")
# trainingset <- cor_names(trainingset)
train_x <- trainingset[, c(-1)]
train_y <- factor(trainingset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

testset <- read.csv("bcrp_inhibition_te.csv")
# testset <- cor_names(testset)
test_x <- testset[, c(-1)]
test_y <- factor(testset[, "classification"], levels = c("noninhibitor", "inhibitor")) 

cl <- 6
registerDoParallel(cl) 
getDoParWorkers()

set.seed(123)
seeds <- vector(mode = "list", length = 6) 
for (i in 1:5) seeds[[i]] <- sample.int(1000, 1)
seeds[[6]] <- sample.int(1000, 1) 
k <- 666666

#the best XGB model
set.seed(k)
bestXGBmodel <- caret::train(train_x, train_y, method = "xgbTree", trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                             tuneGrid = data.frame(nrounds = 210, max_depth = 15, eta = 0.0756131466679135, gamma = 0.46200062709637, colsample_bytree = 0.475267963939626, 
                                                   min_child_weight = 0.769156801893376, subsample = 0.740031271164399));
#the best SGB model
set.seed(k)
bestSGBmodel <- caret::train(train_x, train_y, method = "gbm", trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                             tuneGrid = data.frame(n.trees = 179, interaction.depth = 35, n.minobsinnode = 26, shrinkage = 0.0932939021771154));
#the best SVM model
set.seed(k)
bestSVMmodel <- caret::train(train_x, train_y, method = "svmRadial", trControl = trainControl(method = "cv", number = 5, classProbs = T, savePredictions = "final", seeds = seeds), 
                             tuneGrid = data.frame(C = 6.08098769064484, sigma = 0.0130006814137995));

# explainers
# xgb
explainer_xgb <- DALEX::explain(model = bestXGBmodel, data = test_x,  y = as.numeric(test_y), predict_function = custom_predict_,
                                residual_function = residual_function_, label = "xgb")
# sgb
explainer_sgb <- DALEX::explain(model = bestSGBmodel, data = test_x,  y = as.numeric(test_y), predict_function = custom_predict_,
                                residual_function = residual_function_, label = "sgb")
# svm
explainer_svm <- DALEX::explain(model = bestSVMmodel, data = test_x,  y = as.numeric(test_y), predict_function = custom_predict_,
                                residual_function = residual_function_, label = "svm")

# model performance
# xgb
mp_xgb <- model_performance_(explainer_xgb)
mp_sgb <- model_performance_(explainer_sgb)
mp_svm <- model_performance_(explainer_svm)


# model performance statistics
# dnn
dnn_mp_sta <- as.data.frame(array(, dim = c(559,3)))
names(dnn_mp_sta) <- c("cross_entropy","percentage","model")
x <- sort(mp_dnn$diff)
y <- seq(558,0,-1)/559
dnn_mp_sta$cross_entropy <- x
dnn_mp_sta$percentage <- y
dnn_mp_sta$model <- mp_dnn$label 

# xgb
xgb_mp_sta <- as.data.frame(array(, dim = c(559,3)))
names(xgb_mp_sta) <- c("cross_entropy","percentage","model")
x <- sort(mp_xgb$diff)
y <- seq(558,0,-1)/559
xgb_mp_sta$cross_entropy <- x
xgb_mp_sta$percentage <- y
xgb_mp_sta$model <- mp_xgb$label

# sgb
sgb_mp_sta <- as.data.frame(array(, dim = c(559,3)))
names(sgb_mp_sta) <- c("cross_entropy","percentage","model")
x <- sort(mp_sgb$diff)
y <- seq(558,0,-1)/559
sgb_mp_sta$cross_entropy <- x
sgb_mp_sta$percentage <- y
sgb_mp_sta$model <- mp_sgb$label

# svm
svm_mp_sta <- as.data.frame(array(, dim = c(559,3)))
names(svm_mp_sta) <- c("cross_entropy","percentage","model")
x <- sort(mp_svm$diff)
y <- seq(558,0,-1)/559
svm_mp_sta$cross_entropy <- x
svm_mp_sta$percentage <- y
svm_mp_sta$model <- mp_svm$label

mp_sta <- rbind(dnn_mp_sta, xgb_mp_sta, sgb_mp_sta, svm_mp_sta)
mp_sta$model[mp_sta$model == 'svm'] <- "SVM"
mp_sta$model[mp_sta$model == 'dnn'] <- "DNN"
mp_sta$model[mp_sta$model == 'sgb'] <- "SGB"
mp_sta$model[mp_sta$model == 'xgb'] <- "XGBoost"

# write out residual results on the test set
tmp <- as.data.frame(array(, dim = c(559,0)))
tmp$res_svm <- mp_sta$cross_entropy[mp_sta$model == 'SVM']
tmp$res_dnn <- mp_sta$cross_entropy[mp_sta$model == 'DNN']
tmp$res_sgb <- mp_sta$cross_entropy[mp_sta$model == 'SGB']
tmp$res_xgb <- mp_sta$cross_entropy[mp_sta$model == 'XGBoost']
write.csv(tmp,"res_dis1.csv")

# ==============================================================
# residual distributions for 5-fold cross-validated training set 
# ==============================================================
# residual_function for 5-fold cross-validated training set (except for DNN) 
residual_function_cv <- function(model){
  res = model$pred[,c("noninhibitor","inhibitor")]
  p <- sapply(seq_along(model$pred$obs), function(i) res[i, model$pred$obs[i]])
  return(-log(pmax(p, 1e-4)))
}
# svm
svm_mp_cv <- as.data.frame(array(, dim = c(2240,0)))
x <- sort(residual_function_cv(bestSVMmodel))
y <- seq(2239,0,-1)/2240
svm_mp_cv$cross_entropy <- x
svm_mp_cv$percentage <- y
svm_mp_cv$model <- "SVM" 

# sgb
sgb_mp_cv <- as.data.frame(array(, dim = c(2240,0)))
x <- sort(residual_function_cv(bestSGBmodel))
y <- seq(2239,0,-1)/2240
sgb_mp_cv$cross_entropy <- x
sgb_mp_cv$percentage <- y
sgb_mp_cv$model <- "SGB"

# xgb
xgb_mp_cv <- as.data.frame(array(, dim = c(2240,0)))
x <- sort(residual_function_cv(bestXGBmodel))
y <- seq(2239,0,-1)/2240
xgb_mp_cv$cross_entropy <- x
xgb_mp_cv$percentage <- y
xgb_mp_cv$model <- "XGBoost"

# dnn
res <- read.csv("cv_holdout_prediction_DeepLearning_model_R_1568964995909_1.csv")

residual_function_cv_dnn <- function(res, y){
  res = res[, c("p0", "p1")]
  
  p <- sapply(seq_along(y), function(i) res[i, y[i]])
  return(-log(pmax(p, 1e-4)))
}
dnn_mp_cv <- as.data.frame(array(, dim = c(2240,0)))
x <- sort(residual_function_cv_dnn(res,as.numeric(bcrp_trainingset$classification)))
y <- seq(2239,0,-1)/2240
dnn_mp_cv$cross_entropy <- x
dnn_mp_cv$percentage <- y
dnn_mp_cv$model <- "DNN"

mp_sta_cv <- rbind(svm_mp_cv,sgb_mp_cv,xgb_mp_cv,dnn_mp_cv)

# write out residual results on the training set
tmp <- as.data.frame(array(, dim = c(2240,0)))
tmp$res_svm <- mp_sta_cv$cross_entropy[mp_sta_cv$model == 'SVM']
tmp$res_dnn <- mp_sta_cv$cross_entropy[mp_sta_cv$model == 'DNN']
tmp$res_sgb <- mp_sta_cv$cross_entropy[mp_sta_cv$model == 'SGB']
tmp$res_xgb <- mp_sta_cv$cross_entropy[mp_sta_cv$model == 'XGBoost']
write.csv(tmp,"res_dis_cv.csv")

# residual plotting for all model
title_style <- element_text(size = 10, face = "bold")
tick_style <- element_text(size = 8, face = "bold")
p_te <- ggplot(data = mp_sta, aes(x = cross_entropy, y = percentage, colour = model)) + geom_line(size = 1.1) + 
  theme(plot.title = element_text(hjust = 0.5),title = element_text(size = 12, face = "bold"), axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.position = c(0.9,0.92), legend.title = element_blank(), legend.key = element_blank(),legend.background = element_blank()) + scale_y_continuous(breaks = seq(0,1,0.2),labels = scales::percent) + scale_x_continuous(breaks = seq(0,9.5,0.5))
p1 <- p_te + labs(title = "Test Set",x = "Cross Entropy", y = "Percentage")

p_cv <- ggplot(data = mp_sta_cv, aes(x = cross_entropy, y = percentage, colour = model)) + geom_line(size = 1.1) + 
  theme(plot.title = element_text(hjust = 0.5),title = element_text(size = 12, face = "bold"), axis.title.x = title_style, axis.title.y = title_style, axis.text.x = tick_style, axis.text.y = tick_style, legend.text = title_style, 
        legend.position = c(0.9,0.92), legend.title = element_blank(), legend.key = element_blank(),legend.background = element_blank()) + scale_y_continuous(breaks = seq(0,1,0.2),labels = scales::percent) + scale_x_continuous(breaks = seq(0,9.5,0.5))
p2 <- p_cv + labs(title = "Training Set (5-fold cross-validation)",x = "Cross Entropy", y = "Percentage")
myplot <- grid.arrange(p2, p1, ncol = 2, nrow = 1)

ggsave('residual_distrbution.tiff', plot = myplot, device = "tiff",  dpi = 300, width = 12, height = 6)


# ********************************
# variable importance exploration
# ********************************
# 
# custom loss function (cross entropy, for DNN)
custom_lf <- function(observed, predicted, p_min = 1e-04){
  p <- sapply(seq_along(observed), function(i) predicted[i, observed[i]])
  sum(-log(pmax(p, p_min)))/length(observed)
}
# vi_dnn <- variable_importance(explainer_dnn, loss_function = custom_lf, n_sample = -1)

# do 10 repetition of the process to guarantee higher repeatability of results
set.seed(66)
vi_dnn_rf10 <- replicate(10, variable_importance(explainer_dnn, loss_function = custom_lf, n_sample = -1), simplify = F)

# statisticals for 10 repetitions
vi_dnn_avg_res <- as.data.frame(array(, dim = c(length(vi_dnn_rf10[[1]]$variable),0)))
temp <- 0
for (i in 1:10) {
  temp = temp + vi_dnn_rf10[[i]][order(vi_dnn_rf10[[i]]$variable),]$dropout_loss
}
vi_dnn_avg_res$variable <- vi_dnn_rf10[[1]][order(vi_dnn_rf10[[1]]$variable),]$variable
vi_dnn_avg_res$avg_dropout_loss <- temp/10

# the index of "_full_model_"
a <- which(vi_dnn_avg_res == "_full_model_")
b <- vi_dnn_avg_res[c(-a),][order(vi_dnn_avg_res[c(-a),]$avg_dropout_loss, decreasing = T),]
vi_dnn_final <- rbind(vi_dnn_avg_res[a,], b)

vi_dnn_final$variable <- as.character(vi_dnn_final$variable)
vi_dnn_final[vi_dnn_final$variable == "logP.o.w.",]$variable <- "logP(o/w)"
vi_dnn_final <- vi_dnn_final[-2,]


# Variable importance
# custom loss function (cross entropy, except for dnn)
custom_lf_ <- function(observed, predicted, p_min = 1e-04){
  p <- sapply(seq_along(observed), function(i) predicted[i, observed[i]])
  sum(-log(pmax(p, p_min)))/length(observed)
}

vi_xgb <- variable_importance(explainer_xgb, loss_function = custom_lf_, n_sample = -1)
vi_sgb <- variable_importance(explainer_sgb, loss_function = custom_lf_, n_sample = -1)
vi_svm <- variable_importance(explainer_svm, loss_function = custom_lf_, n_sample = -1)

#do 10 repetition of the process to guarantee higher repeatability of results
set.seed(66)
vi_xgb_rf10 <- replicate(10, vi_xgb, simplify = F)
set.seed(66)
vi_sgb_rf10 <- replicate(10, vi_sgb, simplify = F)
set.seed(66)
vi_svm_rf10 <- replicate(10, vi_svm, simplify = F)


# statisticals for 10 repetitions
# xgb
vi_xgb_avg_res <- as.data.frame(array(,dim = c(length(vi_xgb_rf10[[1]]$variable),0)))
temp <- 0
for (i in 1:10) {
  temp = temp + vi_xgb_rf10[[i]][order(vi_xgb_rf10[[i]]$variable),]$dropout_loss
}
vi_xgb_avg_res$variable <- vi_xgb_rf10[[1]][order(vi_xgb_rf10[[1]]$variable),]$variable
vi_xgb_avg_res$avg_dropout_loss <- temp/10

# the index of "_full_model_"
a <- which(vi_xgb_avg_res == "_full_model_")
b <- vi_xgb_avg_res[c(-a),][order(vi_xgb_avg_res[c(-a),]$avg_dropout_loss, decreasing = T),]
vi_xgb_final <- rbind(vi_xgb_avg_res[a,], b)

vi_xgb_final$variable <- as.character(vi_xgb_final$variable)
vi_xgb_final <- vi_xgb_final[-2,]

# sgb
vi_sgb_avg_res <- as.data.frame(array(,dim = c(length(vi_sgb_rf10[[1]]$variable),0)))
temp <- 0
for (i in 1:10) {
  temp = temp + vi_sgb_rf10[[i]][order(vi_sgb_rf10[[i]]$variable),]$dropout_loss
}
vi_sgb_avg_res$variable <- vi_sgb_rf10[[1]][order(vi_sgb_rf10[[1]]$variable),]$variable
vi_sgb_avg_res$avg_dropout_loss <- temp/10

# the index of "_full_model_"
a <- which(vi_sgb_avg_res == "_full_model_")
b <- vi_sgb_avg_res[c(-a),][order(vi_sgb_avg_res[c(-a),]$avg_dropout_loss, decreasing = T),]
vi_sgb_final <- rbind(vi_sgb_avg_res[a,], b)

vi_sgb_final$variable <- as.character(vi_sgb_final$variable)
vi_sgb_final[vi_sgb_final$variable == "PEOE_VSA.4",]$variable <- "PEOE_VSA+4"
vi_sgb_final <- vi_sgb_final[-2,]

# svm
vi_svm_avg_res <- as.data.frame(array(, dim = c(length(vi_svm_rf10[[1]]$variable),0)))
temp <- 0
for (i in 1:10) {
  temp = temp + vi_svm_rf10[[i]][order(vi_svm_rf10[[i]]$variable),]$dropout_loss
}
vi_svm_avg_res$variable <- vi_svm_rf10[[1]][order(vi_svm_rf10[[1]]$variable),]$variable
vi_svm_avg_res$avg_dropout_loss <- temp/10

# the index of "_full_model_"
a <- which(vi_svm_avg_res == "_full_model_")
b <- vi_svm_avg_res[c(-a),][order(vi_svm_avg_res[c(-a),]$avg_dropout_loss, decreasing = T),]
vi_svm_final <- rbind(vi_svm_avg_res[a,], b)

vi_svm_final$variable <- as.character(vi_svm_final$variable)
vi_svm_final <- vi_svm_final[-2,]


# plotting
a <- vi_dnn_final[1:11,]
a$model <- "dnn"

b <- vi_xgb_final[1:11,]
b$model <- "xgb"

c <- vi_sgb_final[1:11,]
c$model <- "sgb"

d <- vi_svm_final[1:11,]
d$model <- "svm"

vi_final <- rbind(a,b,c,d)

write.csv(vi_final, "vi_final.csv")


# ====================
# consensus modeling
# ====================
con1_sts_cv <- as.data.frame(array(, dim = c(2240,0)))
con1_sts_te <- as.data.frame(array(, dim = c(559,0)))

con2_sts_cv <- as.data.frame(array(, dim = c(2240,0)))
con2_sts_te <- as.data.frame(array(, dim = c(559,0)))

# consensus1
# training set (5-fold cross-validation)
con1_sts_cv$svm <- as.numeric(bestSVMmodel$pred[order(bestSVMmodel$pred$rowIndex),]$pred) - 1
con1_sts_cv$xgb <- as.numeric(bestSVMmodel$pred[order(bestXGBmodel$pred$rowIndex),]$pred) - 1

con2_sts_cv$svm <- as.numeric(bestSVMmodel$pred[order(bestSVMmodel$pred$rowIndex),]$inhibitor)
con2_sts_cv$xgb <- as.numeric(bestSVMmodel$pred[order(bestXGBmodel$pred$rowIndex),]$inhibitor)


cv_pred_dnn <- read.csv("cv_holdout_prediction_DeepLearning_model_R_1568964995909_1.csv")
con1_sts_cv$dnn[cv_pred_dnn$p1 >= 0.5] = 1 
con1_sts_cv$dnn[cv_pred_dnn$p1 < 0.5] = 0 
con2_sts_cv$dnn <- cv_pred_dnn$p1

con1_cv_pred = con1_sts_cv$svm + con1_sts_cv$xgb + con1_sts_cv$dnn
con1_sts_cv$pred[con1_cv_pred >= 2] = 1
con1_sts_cv$pred[con1_cv_pred < 2] = 0

con2_cv_pred = (con2_sts_cv$svm + con2_sts_cv$xgb + con2_sts_cv$dnn)/3
con2_sts_cv$pred[con2_cv_pred >= 0.5] = 1
con2_sts_cv$pred[con2_cv_pred < 0.5] = 0

# test set 
con1_sts_te$svm <- as.numeric(predict(bestSVMmodel, test_x)) - 1 
con2_sts_te$svm <- predict(bestSVMmodel, test_x, type = "prob")$inhibitor
con1_sts_te$xgb <- as.numeric(predict(bestXGBmodel, test_x)) - 1 
con2_sts_te$xgb <- predict(bestXGBmodel, test_x, type = "prob")$inhibitor

te_pred_dnn <- h2o.predict(bestDNNmodel, bcrp_testset.hex)
te_pred_dnn <- as.data.frame(te_pred_dnn)
con1_sts_te$dnn[te_pred_dnn$p1 >= 0.5] = 1 
con1_sts_te$dnn[te_pred_dnn$p1 < 0.5] = 0 
con2_sts_te$dnn <- te_pred_dnn$p1

con1_te_pred = con1_sts_te$svm + con1_sts_te$xgb + con1_sts_te$dnn
con1_sts_te$pred[con1_te_pred >= 2] = 1
con1_sts_te$pred[con1_te_pred < 2] = 0

con2_te_pred = (con2_sts_te$svm + con2_sts_te$xgb + con2_sts_te$dnn)/3
con2_sts_te$pred[con2_te_pred >= 0.5] = 1
con2_sts_te$pred[con2_te_pred < 0.5] = 0

# statisticals
# con1 + cv
train_cv.tab <- table(con1_sts_cv$pred, as.numeric(train_y) - 1)  # statisticals of 5-fold cross-validated training set
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));

# con1 + te
test.tab <- table(con1_sts_te$pred, as.numeric(test_y) - 1);  # statisticals of the test set
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));

results.train_cv <- c("consensus1", train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC);
results.test <- c("consensus1", test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC);
results <- rbind(results.train_cv, results.test);colnames(results) <- c("model", "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC");
write.csv(results,"consensus1_results.csv", row.names = T) 

# con2 + cv
train_cv.tab <- table(con2_sts_cv$pred, as.numeric(train_y) - 1)  # statisticals of 5-fold cross-validated training set
train_cv.TP <- train_cv.tab[2, 2];train_cv.FP <- train_cv.tab[2, 1];train_cv.FN <- train_cv.tab[1, 2];train_cv.TN <- train_cv.tab[1, 1];
train_cv.SE <- train_cv.TP/(train_cv.TP + train_cv.FN);train_cv.SP <- train_cv.TN/(train_cv.TN + train_cv.FP);train_cv.GA <- (train_cv.TP + train_cv.TN)/(train_cv.TP + train_cv.FN + train_cv.TN + train_cv.FP);
train_cv.MCC <- as.numeric((train_cv.TP*train_cv.TN - train_cv.FP*train_cv.FN)/sqrt((train_cv.TP + train_cv.FP) %*% (train_cv.TP + train_cv.FN) %*% (train_cv.TN + train_cv.FP) %*% (train_cv.TN + train_cv.FN)));
train_cv.AUC <- roc(as.numeric(train_y) - 1, con2_cv_pred)$auc[1]

# con2 + te
test.tab <- table(con2_sts_te$pred, as.numeric(test_y) - 1);  # statisticals of the test set
test.TP <- test.tab[2, 2];test.FP <- test.tab[2, 1];test.FN <- test.tab[1, 2];test.TN <- test.tab[1, 1];
test.SE <- test.TP/(test.TP + test.FN);test.SP <- test.TN/(test.TN + test.FP);test.GA <- (test.TP + test.TN)/(test.TP + test.FN + test.TN + test.FP);
test.MCC <- as.numeric((test.TP*test.TN - test.FP*test.FN)/sqrt((test.TP + test.FP) %*% (test.TP + test.FN) %*% (test.TN + test.FP) %*% (test.TN + test.FN)));
test.AUC <- roc(as.numeric(test_y) - 1, con2_te_pred)$auc[1] 

results.train_cv <- c("consensus2", train_cv.TP, train_cv.FP, train_cv.FN, train_cv.TN, train_cv.SE, train_cv.SP, train_cv.GA, train_cv.MCC, train_cv.AUC);
results.test <- c("consensus2", test.TP, test.FP, test.FN, test.TN, test.SE, test.SP, test.GA, test.MCC, test.AUC);
results <- rbind(results.train_cv, results.test);colnames(results) <- c("model", "TP", "FP", "FN", "TN", "SE", "SP", "GA", "MCC", "AUC");
write.csv(results,"consensus2_results.csv", row.names = T)
#****************************************************************model_interpretation.R*********************************************************************#





